@@ -1,12 +1,14 @@
+from __future__ import unicode_literals
+
 import os
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
+from io import BytesIO, StringIO, UnsupportedOperation
 
-from django.utils.encoding import smart_str, smart_unicode
+from django.utils.encoding import smart_text
 from django.core.files.utils import FileProxyMixin
+from django.utils import six
+from django.utils.encoding import force_bytes, python_2_unicode_compatible
 
+@python_2_unicode_compatible
 class File(FileProxyMixin):
     DEFAULT_CHUNK_SIZE = 64 * 2**10
 
@@ -15,20 +17,21 @@ class File(FileProxyMixin):
         if name is None:
             name = getattr(file, 'name', None)
         self.name = name
-        self.mode = getattr(file, 'mode', None)
+        if hasattr(file, 'mode'):
+            self.mode = file.mode
 
     def __str__(self):
-        return smart_str(self.name or '')
-
-    def __unicode__(self):
-        return smart_unicode(self.name or u'')
+        return smart_text(self.name or '')
 
     def __repr__(self):
         return "<%s: %s>" % (self.__class__.__name__, self or "None")
 
-    def __nonzero__(self):
+    def __bool__(self):
         return bool(self.name)
 
+    def __nonzero__(self):      # Python 2 compatibility
+        return type(self).__bool__(self)
+
     def __len__(self):
         return self.size
 
@@ -36,8 +39,13 @@ class File(FileProxyMixin):
         if not hasattr(self, '_size'):
             if hasattr(self.file, 'size'):
                 self._size = self.file.size
-            elif os.path.exists(self.file.name):
+            elif hasattr(self.file, 'name') and os.path.exists(self.file.name):
                 self._size = os.path.getsize(self.file.name)
+            elif hasattr(self.file, 'tell') and hasattr(self.file, 'seek'):
+                pos = self.file.tell()
+                self.file.seek(0, os.SEEK_END)
+                self._size = self.file.tell()
+                self.file.seek(pos)
             else:
                 raise AttributeError("Unable to determine the file's size.")
         return self._size
@@ -59,14 +67,16 @@ class File(FileProxyMixin):
         if not chunk_size:
             chunk_size = self.DEFAULT_CHUNK_SIZE
 
-        if hasattr(self, 'seek'):
+        try:
             self.seek(0)
-        # Assume the pointer is at zero...
-        counter = self.size
+        except (AttributeError, UnsupportedOperation):
+            pass
 
-        while counter > 0:
-            yield self.read(chunk_size)
-            counter -= chunk_size
+        while True:
+            data = self.read(chunk_size)
+            if not data:
+                break
+            yield data
 
     def multiple_chunks(self, chunk_size=None):
         """
@@ -84,7 +94,7 @@ class File(FileProxyMixin):
         # Iterate over this file-like object by newlines
         buffer_ = None
         for chunk in self.chunks():
-            chunk_buffer = StringIO(chunk)
+            chunk_buffer = BytesIO(chunk)
 
             for line in chunk_buffer:
                 if buffer_:
@@ -118,21 +128,29 @@ class File(FileProxyMixin):
     def close(self):
         self.file.close()
 
+@python_2_unicode_compatible
 class ContentFile(File):
     """
     A File-like object that takes just raw content, rather than an actual file.
     """
-    def __init__(self, content):
-        content = content or ''
-        super(ContentFile, self).__init__(StringIO(content))
+    def __init__(self, content, name=None):
+        if six.PY3:
+            stream_class = StringIO if isinstance(content, six.text_type) else BytesIO
+        else:
+            stream_class = BytesIO
+            content = force_bytes(content)
+        super(ContentFile, self).__init__(stream_class(content), name=name)
         self.size = len(content)
 
     def __str__(self):
         return 'Raw content'
 
-    def __nonzero__(self):
+    def __bool__(self):
         return True
 
+    def __nonzero__(self):      # Python 2 compatibility
+        return type(self).__bool__(self)
+
     def open(self, mode=None):
         self.seek(0)
 
